#include <Arduino.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128  // OLED display width, in pixels
#define SCREEN_HEIGHT 64  // OLED display height, in pixels
#define OLED_RESET    -1   // Reset pin (or -1 if sharing Arduino reset pin)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

#define FLOW_SENSOR_PIN 13 // Pin where the flow sensor is connected

volatile uint32_t pulseCount = 0;
uint32_t lastPulseCount = 0;
uint32_t lastTime = 0;
uint32_t autoResetTime = 0; // Tracks time for auto-reset
float totalMilliliters = 0.0;

const float calibrationFactor = 380.00; // Adjust based on real sensor readings

void IRAM_ATTR pulseCounter() {
  pulseCount++;
}

void setup() {
  Serial.begin(115200);
  pinMode(FLOW_SENSOR_PIN, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(FLOW_SENSOR_PIN), pulseCounter, FALLING);

  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Initialize with I2C address 0x3C
    Serial.println(F("SSD1306 allocation failed"));
    for (;;);
  }

  // Display "POURfect" splash screen
  display.display();
  delay(2000);
  display.clearDisplay();
  display.setTextSize(2); // Text size for the splash screen
  display.setTextColor(SSD1306_WHITE);

  display.setCursor(0, 20);
  display.print("POURfect");
  display.display();
  delay(2000);
  display.clearDisplay();
}

void loop() {
  uint32_t currentTime = millis();

  if (currentTime - lastTime >= 1000) { // Update readings every second
    uint32_t pulses = pulseCount - lastPulseCount;
    lastPulseCount = pulseCount;
    lastTime = currentTime;

    float milliliters = (pulses / calibrationFactor) * 1000.0; // Convert to milliliters
    totalMilliliters += milliliters;

    float flowRate = milliliters / 1.0; // Flow rate in ml/s

    // Display "Poured" with larger font
    display.clearDisplay();
    display.setTextSize(2); // Larger font for "Poured" label
    display.setCursor(0, 0);
    display.print("Poured:");
    
    display.setCursor(0, 20);
    display.print(totalMilliliters, 1); // Larger font for "Poured" value
    display.print(" ml");

    // Display "Flow" with medium font
    display.setTextSize(1.5); // Medium font for "Flow" label and value
    display.setCursor(0, 40);
    display.print("Flow: ");
    display.print(flowRate, 1); // Flow rate with 1 decimal place
    display.print(" ml/s");

    display.display();

    // Debugging via Serial
    Serial.print("Poured: ");
    Serial.print(totalMilliliters, 1);
    Serial.println(" ml");
    Serial.print("Flow: ");
    Serial.print(flowRate, 1);
    Serial.println(" ml/s");

    // Update auto-reset timer
    autoResetTime = currentTime;
  }

  // Auto-reset after 30 seconds of inactivity
  if (currentTime - autoResetTime >= 30000) {
    totalMilliliters = 0.0;
    pulseCount = 0;
    lastPulseCount = 0;
    autoResetTime = currentTime;

    display.clearDisplay();
    display.setCursor(0, 20);
    display.print("Ready...");
    display.display();
  }
}
